@@ -179,9 +179,19 @@ public:
         return 0;
     }
 
-    static bool isPropertyField( const TString& fieldName ) {
-        return ( idFieldName() != fieldName &&
-                 textFieldName() != fieldName );
+
+    /**
+     * A doc is empty if it only contains the fields id and text
+     */
+    static bool docEmpty( lucene::document::Document* doc ) {
+        int cnt = 0;
+        lucene::document::DocumentFieldEnumeration* it = doc->fields();
+        while ( it->hasMoreElements() ) {
+            it->nextElement();
+            ++cnt;
+        }
+        delete it;
+        return cnt <= 2;
     }
 
     lucene::document::Document* getDocument( const Node& resource ) {
@@ -210,7 +220,7 @@ public:
                 lucene::document::DocumentFieldEnumeration* fields = oldDoc->fields();
                 while ( fields->hasMoreElements() ) {
                     lucene::document::Field* field = fields->nextElement();
-                    if ( Private::isPropertyField( TString( field->name(), true ) ) ) {
+                    if ( Index::isPropertyField( TString( field->name(), true ) ) ) {
                         docWrapper.addProperty( field->name(), field->stringValue() );
                     }
                 }
@@ -245,15 +255,14 @@ public:
         for ( QHash<Node, lucene::document::Document*>::iterator it = documentCache.begin();
               it != documentCache.end(); ++it ) {
             lucene::document::Document* doc = it.value();
-            getIndexWriter()->addDocument( doc );
+            // never add empty docs
+            if ( !docEmpty( doc ) ) {
+                getIndexWriter()->addDocument( doc );
+            }
             delete( doc );
         }
 
         documentCache.clear();
-
-        // write all changes back to disk
-        getIndexWriter()->optimize();
-        closeWriter();
     }
 };
 
@@ -263,7 +272,7 @@ Soprano::Index::CLuceneIndex::CLuceneIndex( lucene::analysis::Analyzer* analyzer
 {
     d->analyzer = analyzer;
     if ( !d->analyzer ) {
-        d->analyzer = _CLNEW lucene::analysis::SimpleAnalyzer();
+        d->analyzer = _CLNEW lucene::analysis::standard::StandardAnalyzer();
         d->deleteAnalyzer = true;
     }
 }
@@ -533,7 +542,7 @@ Soprano::Iterator<Soprano::Index::QueryHit> Soprano::Index::CLuceneIndex::search
         return hits;
     }
     catch( CLuceneError& err ) {
-        qDebug() << "search failed: " << err.what();
+        qDebug() << "search" << query << "failed: " << err.what();
         setError( exceptionToError( err ) );
         return Iterator<QueryHit>();
     }
@@ -562,7 +571,7 @@ Soprano::Iterator<Soprano::Index::QueryHit> Soprano::Index::CLuceneIndex::search
         }
     }
     else {
-        setError( "Query object is null" );
+        setError( "Invalid CLucene query" );
         return Iterator<QueryHit>();
     }
 }
@@ -653,10 +662,13 @@ void Soprano::Index::CLuceneIndex::dump( QTextStream& s ) const
             while ( e->hasMoreElements() ) {
                 lucene::document::Field* field = e->nextElement();
 
-                s << TString( field->name(), true ).toQString() << ": " << TString( field->stringValue(), true ).toQString() << endl;
+                s << TString( field->name(), true ).toQString() << ": "
+                  << TString( field->stringValue(), true ).toQString()
+                  << endl;
             }
             s << endl;
             _CLDELETE( e );
+            _CLDELETE( doc );
         }
 
         s << flush;
@@ -672,3 +684,13 @@ QString Soprano::Index::CLuceneIndex::defaultSearchField()
 {
     return textFieldName().toQString();
 }
+
+
+void Soprano::Index::CLuceneIndex::clear()
+{
+    int numDocs = d->getIndexReader()->numDocs();
+    for ( int i = 0; i < numDocs; ++i ) {
+        d->getIndexReader()->deleteDocument( i );
+    }
+    d->closeReader();
+}
