@@ -27,6 +27,7 @@
 #include "../soprano/serializer.h"
 
 #include "../server/tcpclient.h"
+#include "../server/localsocketclient.h"
 #include "../server/dbus/dbusclient.h"
 #include "../server/dbus/dbusmodel.h"
 
@@ -114,7 +115,7 @@ static Soprano::Node parseNode( const QString& s )
 }
 
 
-static bool importFile( Soprano::Model* model, const QString& fileName, const QString& serialization )
+static int importFile( Soprano::Model* model, const QString& fileName, const QString& serialization )
 {
     const Soprano::Parser* parser = Soprano::PluginManager::instance()->discoverParserForSerialization( Soprano::mimeTypeToSerialization( serialization ), serialization );
 
@@ -158,7 +159,7 @@ static bool importFile( Soprano::Model* model, const QString& fileName, const QS
 }
 
 
-static bool exportFile( Soprano::Model* model, const QString& fileName, const QString& serialization )
+static bool exportFile( Soprano::StatementIterator data, const QString& fileName, const QString& serialization )
 {
     const Soprano::Serializer* serializer = Soprano::PluginManager::instance()->discoverSerializerForSerialization( Soprano::mimeTypeToSerialization( serialization ), serialization );
 
@@ -167,30 +168,26 @@ static bool exportFile( Soprano::Model* model, const QString& fileName, const QS
         if ( !file.open( QIODevice::WriteOnly ) ) {
             QTextStream s( stderr );
             s << "Could not open file for writing: " << fileName << endl;
-            delete model;
-            return 1;
+            return false;
         }
 
         QTextStream s( &file );
 
-        if ( serializer->serialize( model->listStatements(), s, Soprano::mimeTypeToSerialization( serialization ), serialization ) ) {
+        if ( serializer->serialize( data, s, Soprano::mimeTypeToSerialization( serialization ), serialization ) ) {
             QTextStream s( stdout );
             s << "Successfully exported model." << endl;
-            delete model;
-            return 0;
+            return true;
         }
         else {
             QTextStream s( stderr );
             s << "Failed to export statements: " << serializer->lastError() << endl;
-            delete model;
-            return 2;
+            return false;
         }
     }
     else {
         QTextStream s( stderr );
         s << "Could not find serializer plugin for serialization " << serialization << endl;
-        delete model;
-        return 1;
+        return false;
     }
 }
 
@@ -289,7 +286,7 @@ int version()
 {
     QTextStream s( stdout );
     s << "sopranocmd " << Soprano::versionString() << endl;
-    s << "   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>" << endl;
+    s << "   Copyright (C) 2007-2008 Sebastian Trueg <trueg@kde.org>" << endl;
     s << "   This program is free software; you can redistribute it and/or modify" << endl
       << "   it under the terms of the GNU General Public License as published by" << endl
       << "   the Free Software Foundation; either version 2 of the License, or" << endl
@@ -317,6 +314,7 @@ int usage( const QString& error = QString() )
     s << "Usage:" << endl
       << "   sopranocmd --backend [--dir <storagedir>] [--serialization <s>] <command> [<parameters>]" << endl
       << "   sopranocmd --port <port> [--host <host>] --model <name> [--serialization <s>] <command> [<parameters>]" << endl
+      << "   sopranocmd --socket <socketpath>  --model <name> [--serialization <s>] <command> [<parameters>]" << endl
       << "   sopranocmd --dbus <dbusservice> --model <name> [--serialization <s>] <command> [<parameters>]" << endl
       << endl
       << "   --version           Print version information." << endl
@@ -341,6 +339,9 @@ int usage( const QString& error = QString() )
       << "   --host <host>       Specify the host the Soprano server is running on (defaults to localhost)." << endl
       << "                       (only applicable when querying against the Soprano server.)" << endl
       << endl
+      << "   --socket <path>     Specify the path to the local socket the Soprano server is running on." << endl
+      << "                       (only applicable when querying against the Soprano server.)" << endl
+      << endl
       << "   --serialization <s> The serialization used for commands 'export' and 'import'. Defaults to 'application/x-nquads'." << endl
       << "                       (only applicable with the mentioned commands.)" << endl
       << "                       (be aware that Soprano can understand simple string identifiers such as 'trig' or 'n-triples'." << endl
@@ -353,7 +354,9 @@ int usage( const QString& error = QString() )
       << "                       - For command 'query' this is a SPARQL query string." << endl
       << "                       - For commands 'add' and 'remove' this is a list of 3 or 4 RDF node definitions." << endl
       << "                       - For command 'list' this is an optional list of one to four node definitions." << endl
-      << "                       - For commands 'import' and 'export' this is a local file name to either parse or write to." << endl << endl;
+      << "                       - For commands 'import' and 'export' this is a local file name to either parse or write to." << endl
+      << "                         For command 'export' an optional second parameter before the filename can define a construct" << endl
+      << "                         query to only select a subset to export." << endl << endl;
 
     s << "   Nodes are defined in an N-Triples-like notation:" << endl
       << "   - Resouce nodes are defined in angle brackets." << endl
@@ -389,6 +392,7 @@ int main( int argc, char *argv[] )
     allowedCmdLineArgs.insert( "dir", true );
     allowedCmdLineArgs.insert( "port", true );
     allowedCmdLineArgs.insert( "host", true );
+    allowedCmdLineArgs.insert( "socket", true );
     allowedCmdLineArgs.insert( "dbus", true );
     allowedCmdLineArgs.insert( "serialization", true );
     allowedCmdLineArgs.insert( "querylang", true );
@@ -409,6 +413,7 @@ int main( int argc, char *argv[] )
         if ( args.hasSetting( "port" ) ||
              args.hasSetting( "dbus" ) ||
              args.hasSetting( "host" ) ||
+             args.hasSetting( "socket" ) ||
              args.hasSetting( "model" ) ) {
             return usage( "Cannot mix server parameters with --backend." );
         }
@@ -431,6 +436,7 @@ int main( int argc, char *argv[] )
     QTextStream errStream(stderr);
     Soprano::Client::TcpClient* tcpClient = 0;
     Soprano::Client::DBusClient* dbusClient = 0;
+    Soprano::Client::LocalSocketClient* localSocketClient = 0;
     Soprano::Model* model = 0;
     if ( backendName.isEmpty() ) {
         if ( args.hasSetting( "port" ) ) {
@@ -454,6 +460,20 @@ int main( int argc, char *argv[] )
                 return 2;
             }
         }
+        else if ( args.hasSetting( "socket" ) ) {
+            QString socketPath = args.getSetting( "socket" );
+            localSocketClient = new Soprano::Client::LocalSocketClient();
+            if ( !localSocketClient->connect( socketPath ) ) {
+                errStream << "Failed to contact Soprano server through socket at " << socketPath << endl;
+                delete localSocketClient;
+                return 3;
+            }
+            if ( !( model = localSocketClient->createModel( modelName ) ) ) {
+                errStream << "Failed to create Model: " << localSocketClient->lastError() << endl;
+                delete localSocketClient;
+                return 2;
+            }
+        }
         else if ( args.hasSetting( "dbus" ) ) {
             QString dbusService = args.getSetting( "dbus" );
             dbusClient = new Soprano::Client::DBusClient( dbusService );
@@ -503,8 +523,7 @@ int main( int argc, char *argv[] )
 
     int queryTime = 0;
 
-    if ( command == "import" ||
-         command == "export" ) {
+    if ( command == "import" ) {
         if ( firstArg != args.count()-1 ) {
             delete model;
             return usage();
@@ -513,14 +532,42 @@ int main( int argc, char *argv[] )
         QString fileName = args[firstArg];
         QString serialization = args.getSetting( "serialization", "application/x-nquads" );
 
-        if ( command == "import" ) {
-            return importFile( model, fileName, serialization );
+        return importFile( model, fileName, serialization );
+    }
+    else if ( command == "export" ) {
+        QString fileName;
+        QString query;
+
+        if ( firstArg == args.count()-1 ) {
+            fileName = args[firstArg];
+        }
+        else if ( firstArg == args.count()-2 ) {
+            fileName = args[firstArg+1];
+            query = args[firstArg];
         }
         else {
-            return exportFile( model, fileName, serialization );
+            delete model;
+            return usage();
         }
-    }
 
+        QString queryLang = args.getSetting( "querylang", "SPARQL" );
+        QString serialization = args.getSetting( "serialization", "application/x-nquads" );
+
+        bool success = true;
+        if ( query.isEmpty() ) {
+            // export all statements
+            success = exportFile( model->listStatements(), fileName, serialization );
+        }
+        else {
+            success = exportFile( model->executeQuery( query, Soprano::Query::queryLanguageFromString( queryLang ), queryLang ).iterateStatements(),
+                                  fileName,
+                                  serialization );
+        }
+
+        delete model;
+
+        return success ? 0 : 1;
+    }
     else {
         if ( args.hasSetting( "serialization" ) ) {
             return usage( "Parameter --serialization does only make sense for commands 'import' and 'export'" );
@@ -567,8 +614,8 @@ int main( int argc, char *argv[] )
                 n4 = parseNode( args.arg( 4 ) );
             }
 
-            if ( !n1.isResource() && !n1.isEmpty() ) {
-                errStream << "Subject needs to be a resource node." << endl;
+            if ( n1.isLiteral() ) {
+                errStream << "Subject needs to be a resource or blank node." << endl;
                 delete model;
                 return 1;
             }
